@@ -10,22 +10,18 @@ my $autodb = Class::AutoDB->new(
                           -password=>$DBConnector::DB_PASS
                         );
 
-my $joe=new Person(-name=>'Joe',-sex=>'male');
-my $mary=new Person(-name=>'Mary',-sex=>'female');
+
+my $joe=new Person(-name=>'Joe',-sex=>'male',
+		-alias=>'Joe Alias',
+		-hobbies=>['mountain climbing', 'sailing']);
+my $mary=new Person(-name=>'Mary',-sex=>'female',
+		-alias=>'Mary Alias',
+		-hobbies=>['hang gliding']);
 my $bill=new Person(-name=>'Bill',-sex=>'male');
 
-#
 # Set up friends lists
-print "*** assigning joe's friends\n";
 $joe->friends([$mary,$bill]);
-print "*** assigning mary's friends\n";
 $mary->friends([$joe,$bill]);
-print "*** assigning bill's friends\n";
-$bill->friends([$joe,$mary,'a doll named sue']);
-
-# No need to explicitly store the objects.  AutoDB will store them
+$bill->friends([$joe,$mary]);
+# No need to explicitly store the objects. AutoDB willstore them
 # automatically when they are no longer referenced or when the program ends
-
-#$joe->store;
-#$bill->store;
-#$mary->store;
\ No newline at end of file
